import hashlib


def calculate_checksum(file_path):
    hasher = hashlib.md5()
    with open(file_path, "rb") as f:
        for chunk in iter(lambda: f.read(4096), b""):
            hasher.update(chunk)
    return hasher.hexdigest()


# def calculate_checksum(file_path):
#     sha1_hash = hashlib.sha1()
#     with open(file_path, "rb") as file:
#         for chunk in iter(lambda: file.read(4096), b""):
#             sha1_hash.update(chunk)
#     return sha1_hash.hexdigest()
